""" 
S5server
==========
:Author: Christof Hoeke
:Date: 070717
:Description: a simple reStructuredText Server serving S5
:Version: 0.1
:License: This work is licensed under a `Creative Commons License <http://creativecommons.org/licenses/by/3.0/>`_.

* document root for ReST files served is "/rst/", if no path is given the default is "index.txt" (or with set RST_EXT)
* Files should be written in UTF-8 encoding **without BOM** (ByteOrderMark) which is not liked by docutils in general :(
* ReST (files with extension RST_EXT) requests will be converted to S5 HTML presentations on the fly
* .html requests will be checked if RST_EXT file with same name is present, if yes the request will be redirected to the RST_EXT file (and then converted as above)
* other requests use default SimpleHTTPServer behaviour
"""
# imports PORT and RST_EXT for now
from settings import *

PATH = ''

import BaseHTTPServer
import SimpleHTTPServer

import os.path
import StringIO
import sys
import traceback

import docutils.core
import docutils.io                                                                  


class Handler(SimpleHTTPServer.SimpleHTTPRequestHandler):
    
    def do_GET(self):
        """
        RST_EXT files will be transformed to HTML with docutils
        .html request will look for RST_EXT with same name
        """
        p = '/rst%s' % str(self.path)
        if p == '/rst/':
            p = '/rst/index.txt'
        
        # should be generated from RST_EXT?
        if p.endswith('.html'):
            temp = p[:-5] + RST_EXT
            if os.path.isfile(temp[1:]):
                self.send_response(302)
                self.send_header("Location", temp)
                self.end_headers()
                return
                
        if p.endswith(RST_EXT):
            html = output = ''
            try:               
                html = docutils.core.publish_file(
                    source_path=p[1:],
                    writer_name='s5'
                    )
            except:
                output = traceback.format_exc()
                print >>sys.stderr, output
                self.send_response(500)
                self.send_header("Content-Type", "text/plain; charset=utf-8")
            else:
                self.send_response(200)
                self.send_header("Content-Type", "text/html; charset=utf-8")
            print html
            output += html
            self.send_header("Content-Length", len(output))
            self.end_headers()
            self.wfile.write(output)
        else:
            SimpleHTTPServer.SimpleHTTPRequestHandler.do_GET(self)
        
        
def run(server_class=BaseHTTPServer.HTTPServer,
        handler_class=Handler):
    server_address = (PATH, PORT)
    httpd = server_class(server_address, handler_class)
    httpd.serve_forever()


print 'serving on (localhost) %s:%s' % (PATH, PORT)
print 'stop with CTRL + PAUSE'
run()

